@@ -307,6 +307,8 @@ mhingebst <- function(x,y, cost=NULL, family = c("hinge"), ctrl = bst_control(),
     m <- m + 1
   }
   ensemble <- xselect
+  xselect <- sort(unique(unlist(xselect)))
+  xselect <- xselect[!is.na(xselect)]
   RET <- list(y=y,x=oldx, family = family, learner=learner, k=k, yhat=Fboost, offset=offset, ens=ens, control.tree=control.tree, risk=risk, ctrl = list(center=center, mstop=mstop,nu=nu, df=df), xselect=xselect, coef = coef, ensemble=ensemble, baseclass=baseclass)
   RET$call <- call
   class(RET) <- "mhingebst"
@@ -391,7 +393,7 @@ predict.mhingebst <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=
     split(sample(1:n), rep(1:folds, length = n))
   }
 "cv.mhingebst" <-
-  function(x, y, balance=FALSE, K = 10, cost = NULL, family = "hinge", learner = c("tree","ls", "sm"), ctrl = bst_control(), type = c("risk", "misc"), plot.it = TRUE, se = TRUE, ...)
+  function(x, y, balance=FALSE, K = 10, cost = NULL, family = "hinge", learner = c("tree","ls", "sm"), ctrl = bst_control(), type = c("risk", "misc"), plot.it = TRUE, se = TRUE, n.cores=2, ...)
   {
     call <- match.call()
     family <- match.arg(family)
@@ -410,9 +412,9 @@ predict.mhingebst <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=
     else all.folds <- cv.folds(length(y), K)
     fraction <- seq(mstop)
     residmat <- matrix(NA, mstop, K)
-    for(i in seq(K)) {
-      if(trace)
-        cat("\n CV Fold", i, "\n\n")
+    registerDoParallel(cores=n.cores)
+    i <- 1  ###needed to pass R CMD check with parallel code below
+    residmat <- foreach(i=seq(K), .combine=cbind) %dopar% {
       omit <- all.folds[[i]]
       if(ctrl$twinboost)
         ctrl.cv$f.init <- ctrl$f.init[ - omit, ]
@@ -422,11 +424,16 @@ predict.mhingebst <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=
       else if(type=="misc")
         fit <- predict.mhingebst(fit, newdata = x[omit,  ,drop=FALSE], newy=y[ omit], mstop = mstop, type="error")
       residmat[, i] <- fit
+      residmat[, i]   ### return values for parallel computing only 12/17/2015
     }
     cv <- apply(residmat, 1, mean)
     cv.error <- sqrt(apply(residmat, 1, var)/K)
     object<-list(residmat=residmat, mstop = fraction, cv = cv, cv.error = cv.error)
-    if(plot.it) plotCVbst(object,se=se)
+    if(plot.it){
+	         if(type=="risk") ylab <- "Cross-validation loss values"
+		      else  if(type=="misc") ylab <- "Cross-validation misclassification errors"
+		           plotCVbst(object,se=se, ylab=ylab)
+		   }
     invisible(object)
   }
 
@@ -437,9 +444,7 @@ predict.mhingebst <- function(object, newdata=NULL, newy=NULL, mstop=NULL, type=
     cv.error <- cv.bst.object$cv.error
     family <- cv.bst.object$family
     if(is.null(ylab)){
-      if(family=="hinge")
-      ylab <- "Cross-validation Misclassification Error"
-      else ylab <- "Cross-validation MSE"
+      ylab <- "Cross-validation loss values"
 }
     plot(mstop, cv, type = "b", xlab = "Iteration", ylab= ylab, ylim = range(cv, cv + cv.error, cv - cv.error), main=main)
     if(se)
@@ -543,3 +548,16 @@ fpartial.mhingebst <- function (object, mstop=NULL, newdata=NULL)
   }
   lp 
 }
+
+nsel <- function(object, mstop){
+        if(!class(object) %in% c("mhingebst", "mbst", "rmbst"))
+	stop("object is not a correct class\n")
+	np <- rep(NA, mstop)
+	tmp <- NULL
+	for(i in 1:mstop){
+		tmp <- c(tmp, unlist(object$ensemble[[i]]))
+		np[i] <- length(unique(subset(tmp, !is.na(tmp))))
+	}
+	np
+}
+
